package com.uas.ps.inquiry.controller;

import com.uas.ps.inquiry.entity.*;
import com.uas.ps.inquiry.model.InquiryEnRemind;
import com.uas.ps.inquiry.model.PublicInquiryItem;
import com.uas.ps.inquiry.service.AttachService;
import com.uas.ps.inquiry.service.InquiryForSaleService;
import com.uas.ps.inquiry.service.PublicInquiryService;
import javassist.NotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;


/**
 * Erp从公共服务下载数据接口
 * <p>
 * 便于将之前PublicInquiryController中的Erp相关方法整理到这个Controller，当前Controller路径与之相同。
 *
 * @author dongbw
 * @version 2018年7月3日 14:27:25
 */
@RestController
@RequestMapping("/inquiry/public")
public class ErpDownloadController {

    @Autowired
    private PublicInquiryService publicInquiryService;

    @Autowired
    private AttachService attachService;

    @Autowired
    private InquiryForSaleService saleService;

    private final static Logger log = LoggerFactory.getLogger(ErpDownloadController.class);

    /**
     * 将买家ERP的公共询价写到公共询价服务中心
     *
     * @param inquiries 公共询价信息
     * @param enuu      询价企业UU
     * @param address   询价企业地址信息
     * @return
     * @throws UnsupportedEncodingException
     */
    @RequestMapping(method = RequestMethod.POST)
    public void saveInquiries(@RequestBody List<BatchInquiry> inquiries, Long enuu, String address) throws NotFoundException, UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public post, ERP上传公共询价, size: {}, enuu: {}, 报价的应用来源: {}", inquiries.size(), enuu, InquirySource.ERP.name());
        publicInquiryService.save(inquiries, enuu, URLDecoder.decode(address, "UTF-8"));
        log.info("/inquiry/public post, 耗时:{}", (System.currentTimeMillis() - start));
    }

    /**
     * erp的公共询价单提交待审核时，更新公共询价服务中心的单据设置为不能报价
     *
     * @param inquiries 封装的数据信息
     * @param enuu      询价企业UU
     * @throws UnsupportedEncodingException
     */
    @RequestMapping(value = "/check", method = RequestMethod.POST)
    public void updateInquiryStatus(@RequestBody List<BatchInquiry> inquiries, Long enuu) throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/check post, ERP设置公共询价不能报价, size: {}, enuu: {}", inquiries.size(), enuu);
        publicInquiryService.updateStatus(inquiries, enuu);
        log.info("/inquiry/public/check post, 耗时:{}", (System.currentTimeMillis() - start));
    }

    /**
     * （针对客户）获取公共询价服务中心供应商的报价信息
     *
     * @param enuu 询价企业UU
     * @author hejq
     * @date 2018-01-14 15:20
     */
    @RequestMapping(value = "/quotation", method = RequestMethod.GET)
    public List<InquiryDetail> getReply(Long enuu) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/quotation, ERP获取供应商报价信息, enuu: {}, 应用来源: {}", enuu, InquirySource.ERP.name());
        List<InquiryDetail> details = publicInquiryService.findNotUploadReply(enuu);
        log.info("/inquiry/public/quotation, size: {}, 耗时：{}", details.size(),(System.currentTimeMillis() - start));
        return details;
    }

    /**
     * 公共询价服务中心的报价信息传到买家ERP之后，修改公共询价服务中心里面的询价单明细的上传状态
     *
     * @param data 传回的id串
     * @return
     * @throws UnsupportedEncodingException
     */
    @RequestMapping(value = "/reply/back", method = RequestMethod.POST)
    public void onReplySuccess(@RequestBody String data, Long enuu) throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/reply/back post, ERP获取报价信息回调, id: {}, enuu: {}, 应用来源: {}", data,  enuu, InquirySource.ERP.name());
        String[] idArray = URLDecoder.decode(data, "UTF-8").split(",");
        publicInquiryService.onReplyUploadSuccess(idArray);
        log.info("/inquiry/public/reply/back post, 耗时: {}", (System.currentTimeMillis() - start));
    }

    /**
     * 买家在收到报价信息后，点击提交按钮后，状态传至公共询价服务中心，至此公共询价服务中心的供应商不能再对这张询价单进行报价
     *
     * @param details 封装的data信息
     * @throws UnsupportedEncodingException
     */
    @RequestMapping(value = "/checking", method = RequestMethod.POST)
    public void checkInquiry(@RequestBody List<InquiryDetail> details, Long enuu) throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/checking post, ERP提交报价信息应用禁止报价, size: {}, enuu: {}, 应用来源: {}", details.size(), enuu, InquirySource.ERP.name());
        publicInquiryService.updateInquiryStatus(details, enuu);
        log.info("/inquiry/public/checking post, 耗时: {}", (System.currentTimeMillis() - start));
    }

    /**
     * 买家ERP（不）采纳了价格之后，修改公共询价服务中心里面的报价信息的状态
     *
     * @return 封装的data信息
     * @throws UnsupportedEncodingException
     */
    @RequestMapping(value = "/reply/decide", method = RequestMethod.POST)
    public void onReplyDecide(@RequestBody List<InquiryDecide> decides, Long enuu) throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/reply/decide post, ERP针对供应报价信息进行（不）采纳操作, size: {}, enuu: {}, 报价的应用来源: {}", decides.size(), enuu, InquirySource.ERP.name());
        publicInquiryService.onReplyDecide(decides, enuu);
        log.info("/inquiry/public/reply/decide post, 耗时: {}", (System.currentTimeMillis() - start));
    }

    /**
     * 买家ERP作废询价单后，修改公共询价服务中心里面的状态
     *
     * @param inquiries 封装的询价信息
     * @return
     * @throws UnsupportedEncodingException
     */
    @RequestMapping(value = "/invalid", method = RequestMethod.POST)
    public void onReplyInvalid(@RequestBody List<Inquiry> inquiries, Long enuu) throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/invalid post, ERP作废公共询价单返回数据到公共服务, size: {},  enuu: {}, 报价的应用来源: {}", inquiries.size(), enuu, InquirySource.ERP.name());
        publicInquiryService.onReplyInvalid(inquiries, enuu);
        log.info("/inquiry/public/invalid post, 耗时: {}", (System.currentTimeMillis() - start));
    }

    /**
     * 买家采纳或拒绝之后，上传到报价方UAS
     *
     * @param enUU 报价企业UU
     * @author dongbw
     * @date 2018年7月3日 15:49:11
     */
    @RequestMapping(value = "/down/decide", method = RequestMethod.GET)
    public List<PublicInquiryItem> downloadDecide(Long enUU) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/down/decide, 买家采纳或拒绝之后，上传到报价方UAS,enuu: {}, ", enUU);
        List<PublicInquiryItem> items = publicInquiryService.findDecideNotUploadReply(enUU);
        log.info("/inquiry/public/down/decide, 买家采纳或拒绝之后，上传到报价方UAS, size: {}, 耗时: {}", items.size(), (System.currentTimeMillis() - start));
        return items;
    }

    /**
     * 买家采纳或拒绝之后，上传到报价方UAS 之后，UAS回调 修改状态
     *
     * @param ids  采纳或拒绝的报价单明细id list
     * @param enUU 报价方企业UU
     * @author dongbw
     * @date 2018年7月3日 15:49:11
     */
    @RequestMapping(value = "/down/decide/back", method = RequestMethod.POST)
    public ModelMap downloadDecideBack(String ids, Long enUU) throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/down/decide/back  post, 买家采纳或拒绝之后，上传到报价方UAS的回调, ids: {}, enuu: {},", ids, enUU);
        String[] idArray = URLDecoder.decode(ids, "UTF-8").split(",");
        List<Long> idList = new ArrayList<>();
        for (String id : idArray) {
            idList.add(Long.valueOf(id));
        }
        ModelMap map = publicInquiryService.downloadDecideBack(idList, enUU);
        log.info("/inquiry/public/down/decide/back  post, 耗时:{}", (System.currentTimeMillis() - start));
        return map;
    }

    /**
     * 将报价方在商城报价的单据同步到报价方UAS
     *
     * @param enUU 报价企业UU
     * @author dongbw
     * @date 2018年7月3日 16：21
     */
    @RequestMapping(value = "/down/reply", method = RequestMethod.GET)
    public List<PublicInquiryItem> downReply(Long enUU) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/down/reply, enuu: {},", enUU);
        List<PublicInquiryItem> details = publicInquiryService.findNotUploadToUas(enUU);
        log.info("/inquiry/public/down/reply, ERP获取供应商报价信息, size: {}, 耗时: {}", details.size(), (System.currentTimeMillis() - start));
        return details;
    }

    /**
     * 将报价方在商城报价的单据同步到报价方UAS 之后 回调 修改状态
     *
     * @param enUU 报价企业UU
     * @param ids  报价单明细id list
     * @author dongbw
     * @date 2018年7月3日 16：21
     */
    @RequestMapping(value = "/down/reply/back", method = RequestMethod.POST)
    public void onReplyDownSuccess(String ids, Long enUU) throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/down/reply/back post , 将报价方在商城报价的单据同步到报价方UAS之后回调修改状态, ids: {}, enuu: {}", ids, enUU);
        String[] idArray = URLDecoder.decode(ids, "UTF-8").split(",");
        publicInquiryService.onReplyDownLoadSuccess(idArray);
        log.info("/inquiry/public/down/reply/back post, 耗时: {}", (System.currentTimeMillis() - start));
    }

    /**
     * UAS获取商城账套下企业商机
     *
     * @param vendUU 报价企业UU
     * @author dongbw
     * @date 2018年7月23日 10:34:54
     */
    @RequestMapping(value = "/enremind", method = RequestMethod.GET)
    public List<InquiryEnRemind> downEnRemind(Long vendUU) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/enremind, UAS获取商城账套下企业商机, vendUU: {}", vendUU);
        List<InquiryEnRemind> details = publicInquiryService.findNotUploadEnRemind(vendUU);
        log.info("/inquiry/public/enremind, UAS获取商城账套下企业商机, size: {}, 耗时: {}", details.size(), (System.currentTimeMillis() - start));
        return details;
    }

    /**
     * UAS获取商城账套下企业商机之后 回调 修改状态
     *
     * @param vendUU 报价企业UU
     * @param ids    报价单明细id list
     * @author dongbw
     * @date 2018年7月23日 10:34:50
     */
    @RequestMapping(value = "/enremind/back", method = RequestMethod.POST)
    public void downEnRemindSuccess(String ids, Long vendUU) throws UnsupportedEncodingException {
        long start = System.currentTimeMillis();
        log.info("/inquiry/public/enremind/back post, UAS获取商城账套下企业商机之后，回调修改状态, ids: {},  vendUU: {},", ids, vendUU);
        String[] idArray = URLDecoder.decode(ids, "UTF-8").split(",");
        publicInquiryService.downEnRemindSuccess(idArray, vendUU);
        log.info("/inquiry/public/enremind/back post, 耗时: {}", (System.currentTimeMillis() - start));
    }
}
