package com.uas.ps.inquiry.controller;

import com.alibaba.fastjson.JSON;
import com.uas.ps.inquiry.entity.InquiryDetailInfo;
import com.uas.ps.inquiry.entity.InquiryProductInfo;
import com.uas.ps.inquiry.model.PurcInquiry;
import com.uas.ps.inquiry.model.PurcInquiryItemInfo;
import com.uas.ps.inquiry.page.PageInfo;
import com.uas.ps.inquiry.page.SearchFilter;
import com.uas.ps.inquiry.service.InquiryService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

/**
 * 针对买家，对询价的操作
 *
 * Created by hejq on 2018-01-17.
 */
@RequestMapping("/inquiry/buyer")
@RestController
public class InquiryForBuyerController {

    @Autowired
    private InquiryService inquiryService;

    private final static Logger log = LoggerFactory.getLogger(InquiryForBuyerController.class);

    /**
     * 客户查询供应商报价详情
     *                  B2B调用
     * @param id 报价主表id
     * @param enuu 企业UU
     * @return
     */
    @RequestMapping(value = "/detail", method = RequestMethod.GET)
    public InquiryDetailInfo findById(Long id, Long enuu) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/detail, 客户查询供应商报价详情, id: {}, enuu: {}, ", id, enuu);
        InquiryDetailInfo detailInfo = inquiryService.findById(id, enuu);
        log.info("/inquiry/buyer/detail, 耗时: {}", (System.currentTimeMillis() - start));
        return detailInfo;
    }

    /**
     * 通过企业UU和分页参数获取已发布的询价信息
     *                          调用： B2C
     * @param pageInfo 分页参数
     * @param searchFilter 过滤条件
     * @return
     */
    @RequestMapping(value = "/inquiryList", method = RequestMethod.GET)
    public Page<PurcInquiry> getInquiry(PageInfo pageInfo, SearchFilter searchFilter) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/inquiryList, 通过企业UU和分页参数获取已发布的询价信息, pageInfo: {}, searchFilter: {}", JSON.toJSON(pageInfo), JSON.toJSON(searchFilter));
        Page<PurcInquiry> page = inquiryService.findByPageInfo(pageInfo, searchFilter);
        page = inquiryService.setQuotedAmount(page);
        log.info("/inquiry/buyer/inquiryList, 耗时: {}", (System.currentTimeMillis() - start));
        return page;
    }

    /**
     * 通过企业UU和分页参数获取已发布的询价明细
     *                          调用： B2C
     * @param pageInfo 分页参数
     * @param searchFilter 过滤条件
     * @param state 过滤状态
     * @param overdue 是否过期 1、过期；0、 未过期
     * @return
     */
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Page<PurcInquiryItemInfo> getInquiryList(PageInfo pageInfo, SearchFilter searchFilter, String state, Integer overdue) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/list, 通过企业UU和分页参数获取已发布的询价明细, pageInfo: {}, searchFilter: {}, state: {}, overdue: {}", JSON.toJSON(pageInfo), JSON.toJSON(searchFilter), state, overdue);
        Page<PurcInquiryItemInfo> page = inquiryService.findTodoByPageInfo(pageInfo, searchFilter, state, overdue);
        log.info("/inquiry/buyer/list, 耗时: {}", (System.currentTimeMillis() - start));
        return page;
    }

    /**
     * 针对客户查询单个物料明细的报价情况
     *                          调用：B2C
     * @param id 来源id
     * @param enuu 当前企业UU
     * @return
     */
    @RequestMapping(value = "/product/detail", method = RequestMethod.GET)
    public InquiryProductInfo findInquiryDetailById(Long id, Long enuu) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/product/detail, 针对客户查询单个物料明细的报价情况, id: {}, enuu: {}, ", id, enuu);
        InquiryProductInfo info = inquiryService.findInquiryDetailById(id, enuu);
        log.info("/inquiry/buyer/product/detail, 耗时: {}", (System.currentTimeMillis() - start));
        return info;
    }

    /**
     * 针对客户查询供应商报价信息
     *                       B2C
     * @param pageInfo 分页信息
     * @param searchFilter 过滤条件
     * @param enUU 客户UU
     * @param userUU 用户UU
     * @return
     */
    @RequestMapping(value = "/quotations", method = RequestMethod.GET)
    public Page<PurcInquiryItemInfo> findQuotationsByPage(PageInfo pageInfo, SearchFilter searchFilter, Long enUU, Long userUU, Short overdue) {
        /* logger.log("公共询价", "客户查询供应商报价信息", null, userUU, enUU, InquirySource.MALL.name());*/
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/quotations, 客户查询供应商报价信息, pageInfo: {}, searchFilter: {}, enUU: {}, userUU: {}, overdue: {}",  JSON.toJSON(pageInfo), JSON.toJSON(searchFilter), enUU, userUU, overdue);
        Page<PurcInquiryItemInfo> page = inquiryService.findQuotationsByPage(pageInfo, searchFilter, enUU, userUU, overdue);
        log.info("/inquiry/buyer/quotations, 耗时: {}", (System.currentTimeMillis() - start));
        return page;
    }

    /**
     * 针对客户单个公共询价，查询供应商报价信息
     *                          B2C
     * @param id 询价明细id
     * @return
     */
    @RequestMapping(value = "/quotation", method = RequestMethod.GET)
    public PurcInquiryItemInfo findQuotationById(Long id) {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/quotation, 针对客户单个公共询价 查询供应商报价信息, id: {}", id);
        PurcInquiryItemInfo itemInfo = inquiryService.findQuotationById(id);
        log.info("/inquiry/buyer/quotation, 耗时: {}", (System.currentTimeMillis() - start));
        return itemInfo;
    }

    /**
     * 作为买家，保存更新公共询价单
     *                         B2B、B2C
     * @param inquiry 询价信息
     * @return
     */
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public PurcInquiry saveInquiry(@RequestBody PurcInquiry inquiry) throws Exception {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/save post, 新增公共询价信息,  inquiry: {}",  JSON.toJSON(inquiry));
        inquiry = inquiryService.saveInquiry(inquiry);
        log.info("/inquiry/buyer/save post, 耗时: {}", (System.currentTimeMillis() - start));
        return inquiry;
    }

    /**
     * 通过明细id对供应商报价进行审核操作
     *                          B2B、B2C
     * @param id 报价明细id
     * @param status 状态
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "/decide", method = RequestMethod.POST)
    public void decideQuote(Long id, Short status, Long enuu, Long useruu) throws Exception {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/decide post, 通过明细id对供应商报价进行审核操作, id: {}, 采纳状态: {}, UserUU: {}, Enuu: {}", id, status, useruu, enuu);
        inquiryService.decideQuote(id, status);
        log.info("/inquiry/buyer/decide post, 耗时: {}", (System.currentTimeMillis() - start));
    }

    /**
     * 通过明细id采纳供应商报价
     *                       B2C
     * @param id 报价明细id
     * @param status 状态
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "/adopt", method = RequestMethod.POST)
    public void adoptQuote(Long id, Short status, Long enuu, Long useruu) throws Exception {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/adopt post , 通过明细id采纳供应商报价, id: {}, 采纳状态: {}, UserUU: {}, Enuu: {}", id, status, useruu, enuu);
        inquiryService.adpotQuote(id, status);
        log.info("/inquiry/buyer/adopt post , 耗时: {}", (System.currentTimeMillis() - start));
    }


    /**
     * 通过明细id拒绝供应商报价
     *                      B2C
     * @param id 报价明细id
     * @param status 状态
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "/refuse", method = RequestMethod.POST)
    public void refuseQuote(Long id, Short status, Long enuu, Long useruu,String refusereason) throws Exception {
        long start = System.currentTimeMillis();
        log.info("/inquiry/buyer/refuse post, 通过明细id拒绝供应商报价, id: {}, 采纳状态: {}, UserUU: {}, Enuu: {}, reson: {}", id, status, useruu, enuu, refusereason);
        inquiryService.refuseQuote(id, status, refusereason);
        log.info("/inquiry/buyer/refuse post, 耗时: {}", (System.currentTimeMillis() - start));
    }


}
