package com.uas.ps.inquiry.dao;

import com.uas.ps.inquiry.model.PublicInquiryItem;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import javax.transaction.Transactional;
import java.util.List;
import java.util.Set;

/**
 * 公共询价报价信息表数据库操作
 *
 * Created by hejq on 2018-01-14.
 */
@Repository
public interface PublicInquiryItemDao extends JpaRepository<PublicInquiryItem, Long>, JpaSpecificationExecutor<PublicInquiryItem> {

    /**
     * 通过企业UU和下载状态查询已报价未传回ERP的单据
     *
     * @param enUU 企业UU
     * @param backStatus 未下载状态202
     * @author hejq
     * @date 2018-01-14 11:49
     * @return
     */
    @Query("from PublicInquiryItem d where d.inquiry.enUU = :enUU and d.backStatus = :backStatus and ifnull(d.source, ' ') = 'ERP'")
    List<PublicInquiryItem> findByEnUUAndBackStatus(@Param("enUU") Long enUU, @Param("backStatus") short backStatus);

    /**
     * 根据传回的id更新下载状态
     *
     * @param ids 询价明细idList
     * @author hejq
     * @date 2018-01-14 11:49
     */
    @Modifying
    @Transactional
    @Query("update PublicInquiryItem i set i.backStatus = 351 where i.id in (:ids)")
    void updateBackStatus(@Param("ids") Set<Long> ids);

    /**
     * 按询价单明细ID更新采纳结果
     *              新增将decideDownStatus设置为202 ，供报价方UAS获取
     *
     * @author hejq
     * @date 2018-01-14 15:03
     * @param agreed 采纳结果
     * @param decideStatus 采纳状态
     * @param id 询价id
     */
    @Modifying(clearAutomatically = true)
    @Query("update PublicInquiryItem p set p.agreed = :agreed, p.decideStatus = :decideStatus, p.decideDownStatus = 202 where p.id = :id")
    @Transactional
    void updateDecideStatusByID(@Param("agreed")Short agreed, @Param("decideStatus") Short decideStatus, @Param("id") Long id);

    /**
     * 通过供应商UU号和来源id查询单据明细
     *
     * @author hejq
     * @date 2018-01-14 14:53
     * @param enuu 供应商UU号
     * @param id 来源id
     * @return
     */
    PublicInquiryItem findByVendUUAndSourceId(Long enuu, Long id);

    /**
     * 通过主表id查询序号
     *
     * @param id 询价主表id
     * @return
     */
    @Query("select count(1) from PublicInquiryItem i where i.inquiry.id = :id")
    Integer countByInquiryId(@Param("id") Long id);

    /**
     * 通过来源id查询报价信息
     *
     * @param sourceId
     * @return
     */
    List<PublicInquiryItem> findBySourceId(Long sourceId);

    /**
     * 通过来源id查询报价信息 报价时间倒序
     *
     * @param sourceId
     * @return
     */
    List<PublicInquiryItem> findBySourceIdOrderByOfferTimeDesc(Long sourceId);

    /**
     * 根据enUU和上传状态获取报价明细
     * @param vendUU 企业UU
     * @param status  状态
     * @return 报价单list
     */
    List<PublicInquiryItem> findByVendUUAndDecideDownStatus(Long vendUU, short status);

    /**
     * 通过企业UU和回复下载状态查询已报价 未传到卖家ERP的单据
     *
     * @param vendUU 企业UU
     * @param replySendStatus 未下载状态202
     * @author dongbw
     * @return
     */
    @Query(value = "select * from public$inquiryitems d where d.id_venduu = :vendUU and d.id_replysendstatus = :replySendStatus limit 200", nativeQuery = true)
    List<PublicInquiryItem> findByVendUUAndReplySendStatus(@Param("vendUU")Long vendUU, @Param("replySendStatus") short replySendStatus);

    /**
     * 查询所有询价明细已报价条数
     * @param itemIds 询价明细idArray
     * @return
     */
    @Query(value = "select count(distinct id_sourceid) from public$inquiryitems where id_sourceid in :itemIds", nativeQuery = true)
    Integer getQuotedAmountBySourceId(@Param("itemIds")Long[] itemIds);

    /**
     * 查询所有询价明细替代料报价数
     * @param itemIds 询价明细idArray
     * @return
     */
    @Query(value = "select count(distinct id_sourceid) from public$inquiryitems where id_sourceid in :itemIds and id_isreplace = 1", nativeQuery = true)
    Integer getReplaceQuotedAmountBySourceId(@Param("itemIds")Long[] itemIds);

    /**
     * 查询某个时间段的报价数量
     * @param startDate
     * @param endDate
     * @return
     */
    @Query(value = "select count(*) from public$inquiryitems where id_offertime <= :endDate && id_offertime >= :startDate", nativeQuery = true)
    Integer countOfferAmount(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 查询某个时间段的报价用户数量
     * @param startDate
     * @param endDate
     * @return
     */
    @Query(value = "select count(DISTINCT(id_venduu)) from public$inquiryitems where id_offertime <= :endDate && id_offertime >= :startDate", nativeQuery = true)
    Integer countOfferEnAmount(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 获取按最低价格计算的合计价格
     * @param itemIds 询价明细Ids
     * @return 合计价格总额
     */
    @Query(value = "select sum(sumprice) from (select min(id_sumprice) as sumprice,id_sourceid as sourceid from public$inquiryitems where id_sourceid in (:itemIds) group by id_sourceid)a", nativeQuery = true)
    Double getMinPriceSumBySourceId(@Param("itemIds") Long[] itemIds);

    /**
     * 获取按最低交期计算的合计价格
     * @param itemIds 询价明细Ids
     * @return 合计价格总额
     */
    @Query(value = "select sum(sumprice) from (select min(id_sumprice) as sumprice from public$inquiryitems,(select min(id_leadtime) as leadtime,id_sourceid as sourceid from public$inquiryitems where id_sourceid in :itemIds group by id_sourceid) a where id_sourceid = a.sourceid and id_leadtime = a.leadtime group by id_sourceid) b", nativeQuery = true)
    Double getMinDeliverySumBySourceId(@Param("itemIds") Long[] itemIds);

    /**
     * 根据上周最后注册用户UU获取老用户报价数量
     * @param userUU  上周最后注册用户uu
     * @param dateParams 日期
     * @return 查询结果
     */
    @Query(value = "select count(1) from public$inquiryitems where date(id_date) = :dateParams and id_venduseruu <= :userUU", nativeQuery = true)
    Integer sumOldReplyCountByUserUUAndDate(@Param("userUU")Integer userUU, @Param("dateParams")String dateParams);

    /**
     * 根据上周最后注册用户UU获取老用户报价数量
     * @param dateParams 日期
     * @return 查询结果
     */
    @Query(value = "select count(1) from public$inquiryitems where date(id_date) = :dateParams ", nativeQuery = true)
    Integer sumReplyCountByDate(@Param("dateParams")String dateParams);

    /**
     * 日期区间内，老客户的报价数量
     * @param startDate 开始时间 yyyy-MM-dd
     * @param endDate 结束时间 yyyy-MM-dd
     * @param userUU 上周注册最大用户UU
     * @return 查询结果
     */
    @Query(value = "select count(1) from public$inquiryitems where date(id_date) between :startDate and :endDate and id_venduseruu <= :userUU", nativeQuery = true)
    Integer sumOldReplyCountWithDateScope(@Param("startDate") String startDate, @Param("endDate") String endDate, @Param("userUU") Integer userUU);

    /**
     * 日期区间内，所有报价价数量
     * @param startDate 开始时间 yyyy-MM-dd
     * @param endDate 结束时间 yyyy-MM-dd
     * @return 查询结果
     */
    @Query(value = "select count(1) from public$inquiryitems where date(id_date) between :startDate and :endDate", nativeQuery = true)
    Integer sumReplyCountWithTimeScope(@Param("startDate") String startDate, @Param("endDate") String endDate);

    /**
     * 根据sourceIds 查找报价单
     * @param vendUU 报价企业uu
     * @param sourceIds 询价明细id串  1，2，3，4
     * @return
     */

    List<PublicInquiryItem> findByVendUUAndSourceIdIn(@Param("vendUU") Long vendUU, @Param("sourceIds") List<Long> sourceIds);
}
