package com.uas.ps.inquiry.service;

import com.uas.ps.inquiry.domain.IPage;
import com.uas.ps.inquiry.entity.*;
import com.uas.ps.inquiry.model.*;
import com.uas.ps.inquiry.page.PageInfo;
import com.uas.ps.inquiry.page.SearchFilter;
import javassist.NotFoundException;
import org.springframework.data.domain.Page;
import org.springframework.ui.ModelMap;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 公共询价接口
 * <p>
 * Created by hejq on 2018-01-13.
 */
public interface PublicInquiryService {

    /**
     * 保存公共询价信息
     *
     * @param inquiries erp传入公共询价的信息
     * @param enuu      询价企业UU号
     * @param address   询价企业地址
     * @author hejq
     * @date 2018-01-14 11:45
     */
    void save(List<BatchInquiry> inquiries, Long enuu, String address) throws NotFoundException;

    /**
     * 将ERP传入的inquiry信息转成公共询价服务中心需要的信息
     *
     * @param inquiries 将ERP传入的inquiry信息
     * @param enuu      询价企业UU号
     * @return
     * @author hejq
     * @date 2018-01-14 11:45
     */
    List<PurcInquiry> covert(List<BatchInquiry> inquiries, Long enuu, String address) throws NotFoundException;

    /**
     * erp提交审核时，公共询价服务中心单据禁用报价功能
     *
     * @param inquiries erp询价信息
     * @param enuu      询价企业UU号
     * @author hejq
     * @date 2018-01-14 11:45
     */
    void updateStatus(List<BatchInquiry> inquiries, Long enuu);

    /**
     * 查询供应商的报价信息
     *
     * @param enuu 企业uu
     * @return
     * @author hejq
     * @date 2018-01-14 11:45
     */
    List<InquiryDetail> findNotUploadReply(Long enuu);

    /**
     * 将平台的公共询价转成ERP的对应的字段
     *
     * @param notUploadReply 未下载的供应商报价信息
     * @return
     * @author hejq
     * @date 2018-01-14 11:51
     */
    List<InquiryDetail> convertPublicInquiryReply(List<PublicInquiryItem> notUploadReply);

    /**
     * 下载完成后更新公共询价服务中心询价单明细的下载状态
     *
     * @param idArray 询价单明细id
     * @author hejq
     * @date 2018-01-14 12:12
     */
    void onReplyUploadSuccess(String[] idArray);

    /**
     * 下载成功之后，更新报价下载状态
     * @param idArray 报价单明细id
     */
    void onReplyDownLoadSuccess(String[] idArray);

    /**
     * ERP提交公共询价单之后，更新平台状态，让公共询价服务中心询价单不能进行报价操作
     *
     * @param inquiries 询价信息
     * @author hejq
     * @date 2018-01-14 12:20
     */
    void updateInquiryStatus(List<InquiryDetail> inquiries, Long enuu);

    /**
     * ERP对报价信息的采纳状态上传到公共询价服务中心
     *
     * @param decides 采纳信息
     * @author hejq
     * @date 2018-01-14 12:24
     */
    void onReplyDecide(List<InquiryDecide> decides, Long enuu);

    /**
     * 将ERP的报价采纳信息转为公共询价服务中心需要的信息
     *
     * @param decides erp报价采纳信息
     * @author hejq
     * @date 2018-01-14 12:26
     */
    List<PublicInquiryItem> convertInquiryDecide(List<InquiryDecide> decides);

    /**
     * 买家ERP作废询价单后，修改公共询价服务中心里面的状态
     *
     * @param inquiries 询价信息
     * @param enuu      企业UU
     * @author hejq
     * @date 2018-01-14 12:28
     */
    void onReplyInvalid(List<Inquiry> inquiries, Long enuu);

    /**
     * 将公共询价列表转成需要报价的询价单，类型是公共询价
     *
     * @param id     询价明细id
     * @param enuu   报价企业UU
     * @param useruu 报价人uu
     * @return
     * @author hejq
     * @date 2018-01-14 14:46
     */
    ModelMap transtoInquiry(Long id, Long enuu, Long useruu);

    /**
     * 通过分页参数和过滤条件查询报价信息
     *
     * @param info   分页参数
     * @param filter 过滤条件
     * @return
     * @author hejq
     * @date 2018-01-17 10:30
     */
    IPage<PublicInquiryItemInfo> findByPageInfo(PageInfo info, SearchFilter filter);

    /**
     * 通过id查询单个已转报价的信息
     *
     * @param id
     * @return
     */
    PublicInquiryItemInfo findOneInfo(Long id, Long enuu, Long useruu);

    /**
     * 商城获取采纳列表
     *
     * @param info
     * @param filter
     * @return
     */
    Page<PublicInquiryItemInfo> findListByPageInfo(PageInfo info, SearchFilter filter);

    /**
     * 通过分页参数和过滤条件查询公共询价价信息
     *
     * @param info
     * @param filter
     * @return
     * @author hejq
     * @date 2018-01-18 15:32
     */
    IPage<PurcInquiryItemInfo> findTodoByPageInfo(PageInfo info, SearchFilter filter);

    /**
     * 根据传入条件封装查询条件
     *
     * @param pageInfo 分页参数
     * @param state    过滤条件
     * @return
     */
    PageInfo covert(PageInfo pageInfo, String state, Short overdue);

    /**
     * 获取公共询价列表信息
     *
     * @param enuu 当前企业UU
     * @param size 大小
     * @return
     */
    List<InquiryMessage> getMessageList(Long enuu, Integer size);

    /**
     * 消息推送
     *
     * @param inquiryItems 询价明细
     * @param sourceApp    应用来源
     * @throws Exception
     */
    void notifyMessage(List<PurcInquiryItem> inquiryItems, String sourceApp) throws Exception;

    /**
     * 针对供应商，客户对报价采纳结果进行消息通知
     *
     * @param item      询价明细
     * @param sourceApp 来源
     */
    void notifyDecide(PublicInquiryItem item, String sourceApp);

    /**
     * 通过id查询询价信息
     *
     * @param id 公共询价主表id
     * @return
     */
    PurcInquiry findInquiryById(Long id);

    /**
     * 商城-采购排行榜
     *
     * @return
     */
    List<Map<String, Object>> findCodeRankingList();

    /**
     * 通过明细id查询询价详情
     *
     * @param itemId 明细id
     * @return
     */
    PurcInquiryItemInfo findItemByItemId(Long itemId, Long enuu);

    /**
     * 将推荐信息转成UU互联需要的数据
     *
     * @param content 推荐信息
     * @return
     */
    List<PurcInquiryItemInfo> covertRemindToInquiry(List<InquiryRemind> content);

    /**
     * 消息推送
     *
     * @param item      询价明细
     * @param sourceApp 应用来源
     * @return
     */
    Page<MessageModel> refuseMessage(PublicInquiryItem item, String sourceApp);

    /**
     * 消息推送
     *
     * @param item      询价明细
     * @param sourceApp 应用来源
     * @return
     */
    Page<MessageModel> adoptMessage(PublicInquiryItem item, String sourceApp);

    /**
     * 针对买家，通知买家供应商已报价
     *
     * @param
     * @param
     * @return
     */
    Page<MessageModel> ReceiveMessage(PublicInquiryItem item, String sourceApp);

    /**
     * 公共询价单明细总数
     *
     * @return
     */
    ModelMap getPurcInquiryItemCount();

    /**
     * 查询某年某月的询价明细总数
     *
     * @param month
     * @return
     */
    ModelMap countByMonth(Integer year, Integer month);


    /**
     * 根据年份和月份字符串获取条数
     *
     * @param year 年份
     * @param months 月份字符串 6,7,8
     * @return map
     */
    List<ModelMap> countByMonths(Integer year, String months);

    /**
     * 根据enUU获取已被采纳或拒绝的报价单明细
     * @param enUU  报价企业UU
     * @return
     */
    List<PublicInquiryItem> findDecideNotUploadReply(Long enUU);

    /**
     * 已被采纳或拒绝的报价单明细 上传买方UAS之后，回调
     * @param ids  报价单明细ids
     * @param enUU 报价企业UU
     * @return
     */
    ModelMap downloadDecideBack(List<Long> ids, Long enUU);

    /**
     * 将在b2c或者b2b报价的单据，上传到卖方UAS
     * @param enUU 企业UU
     * @return
     */
    List<PublicInquiryItem> findNotUploadToUas(Long enUU);

    /**
     * UAS获取商城账套下企业商机
     *
     * @param enUU 报价企业UU
     * @author dongbw
     * @return 待下载的企业商机
     */
    List<InquiryEnRemind> findNotUploadEnRemind(Long enUU);

    /**
     * UAS获取商城账套下企业商机之后 回调 修改状态
     *
     * @param enUU 报价企业UU
     * @param idArray 报价单明细id list
     */
    void downEnRemindSuccess(String[] idArray, Long enUU);

    /**
     * 获取当前月和上个月询价单数量
     * @return 数量map
     */
    ModelMap getCountOfLastAndThisMonth();

    /**
     * 查询单条bom询价单的报价情况
     * @param inquiry  询价单
     * @return 询价单
     */
    PurcInquiry getQuotedDetail(PurcInquiry inquiry);

    /**
     * 根据id设置询价单明细有效状态
     * @param id  询价单明细id purcInquiryItem
     * @param valid 1有效  0 无效
     * @return
     */
    ModelMap deleteItems(Long id, Integer valid);

    /**
     * 商城后台-统计询价产品和用户
     * @param startDate
     * @param endDate
     * @return
     */
    ModelMap getInquiryCountData(String startDate, String endDate);

    /**
     * 查询个人不同型号发布询价数量统计
     * @param enuu 企业uu
     * @param useruu  用户uu
     * @param starttime  开始时间
     * @param endtime  结束时间
     * @return 查询结果
     */
    ModelMap sumPersonalPublish(Long enuu, Long useruu, Long starttime, Long endtime);

    /**
     * 查询企业不同型号发布询价数量统计
     * @param enuu 企业uu
     * @param starttime  开始时间
     * @param endtime  结束时间
     * @return 查询结果
     */
    ModelMap sumEnterprisePublish(Long enuu, Long starttime, Long endtime);

    /**
     * 查询个人报价询价数量统计
     * @param enuu 企业uu
     * @param useruu  用户uu
     * @param starttime  开始时间
     * @param endtime  结束时间
     * @return 查询结果
     */
    ModelMap sumPersonalReply(Long enuu, Long useruu, Long starttime, Long endtime);

    /**
     * 查询企业回复询价数量统计
     * @param enuu 企业uu
     * @param starttime  开始时间
     * @param endtime  结束时间
     * @return 查询结果
     */
    ModelMap sumEnterpriseReply(Long enuu, Long starttime, Long endtime);

    /**
     * 获取今天和昨天的询价和报价统计
     * @param time 时间
     * @return 统计结果
     * @exception Exception 接口异常
     */
    HashMap<String, Object> sumRecentPublishWithTime(Long time) throws Exception;

    /**
     * 获取区间时间内的询价和报价统计
     * @param starttime 起始时间
     * @param endtime 起始时间
     * @return 统计结果
     * @exception Exception 接口异常
     */
    ModelMap sumPublishWithTimeScope(Long starttime, Long endtime) throws Exception;
    
    /**
     * 根据企业uu查询发布
     * @param enuu
     * @return
     */
    ModelMap sumCountWithEnuu(Long enuu);
}

